#include "DigitalPot.h"
#include "Delay.h"

uint8_t DigitalPot::CurrentValue = 0x80; // Power-on default as per spec.

void DigitalPot::Init()
{ 
    // Make sure SS# (PB4) is high initially and configure MISO as an input with the pullup enabled.
    PORTB |= _BV(PB4) | _BV(PB6);

    // SS# (PB4), MOSI (PB5) and SCK (PB7) are outputs.
    DDRB |= _BV(PB4) | _BV(PB5) | _BV(PB7);

    // Set the SPI control register to generate no interrupts, be enabled, use MSB as the data order, 
    // be an SPI master, use SPI mode 0, and use a clock frequency of Fosc/64.
    SPCR = _BV(SPE) | _BV(MSTR) | _BV(SPR1);

    SPSR = 0; // Make sure the SPI2X (Double SPI Speed) bit is cleared.  
}

void DigitalPot::SetWiperValue(uint8_t NewWiperValue)
{
    // The following is most likely unnecessary.  Better safe than sorry.
    _delay_us(0.040); // Per MCP41XXX spec.  "CS# High Time.  Minimum 40ns."

    // Bring SS# (MCP41010's CS# pin) low.
    PORTB &= ~_BV(PB4);

    _delay_us(0.040); // Per MCP41XXX spec.  "CS# Fall to First Rising CLK Edge.  Minimum 40ns."

    SPDR = 0x11; // Command byte.  Write data.  Potentiometer 0.

    // Wait for SPI data to be sent...
    while (!(SPSR & (1 << SPIF)))
        ;
 
    SPDR = NewWiperValue; // Data register value.

    // Wait for SPI data to be sent...
    while (!(SPSR & (1 << SPIF)))
        ;

    _delay_us(0.030); // Per MCP41XXX spec.  "SCK Rise to CS# Rise Hold Time.  Minimum 30ns."

    // Bring SS# high.
    PORTB |= _BV(PB4);

    CurrentValue = NewWiperValue;
}

